#include <stdio.h>
#include <htc.h>
#include "usart.h"
#include "lcd.h"
#include "i2c.h"

__CONFIG(FOSC_XT & WDTE_OFF & PWRTE_ON & MCLRE_ON & CP_OFF & CPD_OFF & BOREN_OFF & CLKOUTEN_ON & IESO_OFF & FCMEN_OFF);

__CONFIG(WRT_OFF & PLLEN_OFF & STVREN_OFF & BORV_25 & LVP_OFF);		//!!! use BORV_HI for latest PICC compiler

#define _XTAL_FREQ  4000000

void init_pic(void)
{
	//OSCCON = 0b01101000;
	PORTA = 0x00;
	PORTB = 0x00;
	ANSELA = 0x00;
	ANSELB = 0b00001000; //analogue input on AN9 (RB3)
	TRISA=0b00000000;
	TRISB=0b00111110;	//set up for I2C and USART, plus ADC input on AN9 (RB3)
}

void init_a2d(void)
{
	ADCON0=0b00100100;	// AD channel 9
	ADCON1=0b01010000;	// left justify, /16 (4uS @ 4MHz), ref VSS, ref VDD
	ADON=1;			// turn on the A2D conversion module
}


void dec_display(unsigned char units, int lcdSerial) //converts 8 bit binary to decimal and displays on LCD or RS232
{
	unsigned char tens;
	unsigned char hundreds;
	
	hundreds = units / 100; 
	units %= 100;
	tens = units / 10;
	units %= 10;

	if (lcdSerial == 0)
	{
	lcd_putch(hundreds + 0x30); //LCD
	lcd_putch(tens + 0x30);
	lcd_putch(units + 0x30);
	}
	else
	{
	printf ("\r");
	//putch(hundreds + 0x30);	//RS232
	putch(tens + 0x30);
	putch(units + 0x30);
	}	
}

void half_display(unsigned char lowByte)
{
	lowByte&=0b11000000;	//clear the least significant 6 bits

	if (lowByte > 127)
	{
	lcd_puts(".5C");
	printf(".5C");
	}
	else
	{
	lcd_puts(".0C");
	printf(".0C");
	}
}

void main(void)
{
	unsigned char x;
	unsigned char y = 0;
	unsigned char data1;
	unsigned char data2;

	init_pic();
	init_a2d();
	lcd_init();
	init_comms();	// set up the USART - settings defined in usart.h
	lcd_goto(0);  // select first line 	
	lcd_puts("Testing");
	lcd_goto(0x40);	// Select second line
	lcd_puts("program");
	printf("\rPicDemo\n");
	__delay_ms(1000);
	lcd_clear();



while (1){
		ADGO=1;			// initiate AD conversion

		i2c_WriteTo(0x90);
		i2c_PutByte(0x00);	/* send data 0x00 */
		i2c_ReadFrom(0x90);
		data1 = i2c_GetByte(I2C_MORE);
		data2 = i2c_GetByte(I2C_LAST);
		lcd_puts("Temp=");
		dec_display(data1,0); //display degrees on LCD
		dec_display(data1,1); //send degrees to serial port
		half_display(data2); //send half degrees to serial port
		lcd_goto(0x40);	// Select second line
		
		//__delay_ms(10);
		lcd_puts("ADC=");
		dec_display(ADRESH, 0);//display AD value on LCD

		lcd_puts(" y=");
		dec_display(y, 0);//display value of y on LCD

		__delay_ms(1000);
		lcd_clear();
		y++;
	}
}
